@@ -19,7 +19,7 @@ void Elens_track_local_particle(ElensData el, LocalParticle* part0){
 
     int const polynomial_order = ElensData_get_polynomial_order(el);
 
-    double const* coefficients_polynomial =
+    /*gpuglmem*/ double const* coefficients_polynomial =
                                 ElensData_getp1_coefficients_polynomial(el, 0);
 
     //start_per_particle_block (part0->part)
