@@ -5,6 +5,7 @@ from scipy.optimize import fsolve, minimize
 
 from .jacobian import jacobian
 from .twiss import TwissInit
+from .general import _print
 import xtrack as xt
 
 class OrbitOnly:
@@ -16,22 +17,22 @@ class OrbitOnly:
         self.zeta = zeta
         self.delta = delta
 
-def _error_for_match(knob_values, vary, targets, tracker, return_scalar,
+def _error_for_match(knob_values, vary, targets, line, return_scalar,
                      call_counter, verbose, tw_kwargs):
 
-    print(f"Matching: twiss call n. {call_counter['n']}       ", end='\r', flush=True)
+    _print(f"Matching: twiss call n. {call_counter['n']}       ", end='\r', flush=True)
     call_counter['n'] += 1
 
     for kk, vv in zip(vary, knob_values):
-        tracker.vars[kk.name] = vv
-    tw = tracker.twiss(**tw_kwargs)
+        line.vars[kk.name] = vv
+    tw = line.twiss(**tw_kwargs)
 
     res_values = []
     target_values = []
     for tt in targets:
         if isinstance(tt.tar, str):
             if tt.at is not None:
-                res_values.append(tw[tt.at, tt.tar])
+                res_values.append(tw[tt.tar, tt.at])
             else:
                 res_values.append(tw[tt.tar])
         else:
@@ -53,14 +54,14 @@ def _error_for_match(knob_values, vary, targets, tracker, return_scalar,
     if np.all(np.abs(err_values) < tols):
         err_values *= 0
         if verbose:
-            print('Found point within tolerance!')
+            _print('Found point within tolerance!')
 
     for ii, tt in enumerate(targets):
         if tt.scale is not None:
             err_values[ii] *= tt.scale
 
     if verbose:
-        print(f'x = {knob_values}   f(x) = {res_values}')
+        _print(f'x = {knob_values}   f(x) = {res_values}')
 
     if return_scalar:
         return np.sum(err_values * err_values)
@@ -97,7 +98,7 @@ class Target:
         self.at = at
         self.scale = scale
 
-def match_tracker(tracker, vary, targets, restore_if_fail=True, solver=None,
+def match_line(line, vary, targets, restore_if_fail=True, solver=None,
                   verbose=False, **kwargs):
 
     if 'twiss_init' in kwargs and kwargs['twiss_init'] is not None:
@@ -106,14 +107,14 @@ def match_tracker(tracker, vary, targets, restore_if_fail=True, solver=None,
             'ele_start must be provided if twiss_init is provided')
         if isinstance(twiss_init, OrbitOnly):
             if not isinstance(kwargs['ele_start'], str):
-                element_name = tracker.line.element_names[kwargs['ele_start']]
+                element_name = line.element_names[kwargs['ele_start']]
             else:
                 element_name = kwargs['ele_start']
-            particle_on_co=tracker.build_particles(
+            particle_on_co=line.build_particles(
                 x=twiss_init.x, px=twiss_init.px,
                 y=twiss_init.y, py=twiss_init.py,
                 zeta=twiss_init.zeta, delta=twiss_init.delta)
-            particle_on_co.at_element = tracker.line.element_names.index(
+            particle_on_co.at_element = line.element_names.index(
                                                                 element_name)
             kwargs['twiss_init'] = TwissInit(
                 particle_on_co=particle_on_co,
@@ -156,7 +157,7 @@ def match_tracker(tracker, vary, targets, restore_if_fail=True, solver=None,
             solver = 'fsolve'
 
     if verbose:
-        print(f'Using solver {solver}')
+        _print(f'Using solver {solver}')
 
     steps = []
     for vv in vary:
@@ -173,9 +174,9 @@ def match_tracker(tracker, vary, targets, restore_if_fail=True, solver=None,
     call_counter = {'n': 0}
     _err = partial(_error_for_match, vary=vary, targets=targets,
                 call_counter=call_counter, verbose=verbose,
-                tracker=tracker, return_scalar=return_scalar, tw_kwargs=kwargs)
+                line=line, return_scalar=return_scalar, tw_kwargs=kwargs)
     _jac= partial(_jacobian, steps=steps, fun=_err)
-    x0 = [tracker.vars[vv.name]._value for vv in vary]
+    x0 = [line.vars[vv.name]._value for vv in vary]
     try:
         if solver == 'fsolve':
             (res, infodict, ier, mesg) = fsolve(_err, x0=x0.copy(),
@@ -216,42 +217,42 @@ def match_tracker(tracker, vary, targets, restore_if_fail=True, solver=None,
             if not info['success']:
                 raise RuntimeError("jacobian failed: %s" % info['message'])
         for vv, rr in zip(vary, res):
-            tracker.vars[vv.name] = rr
+            line.vars[vv.name] = rr
     except Exception as err:
         if restore_if_fail:
             for ii, rr in enumerate(vary):
-                tracker.vars[rr] = x0[ii]
-        print('\n')
+                line.vars[rr] = x0[ii]
+        _print('\n')
         raise err
-    print('\n')
+    _print('\n')
     return result_info
 
-def closed_orbit_correction(tracker, tracker_co_ref, correction_config,
+def closed_orbit_correction(line, line_co_ref, correction_config,
                             solver=None, verbose=False, restore_if_fail=True):
 
     for corr_name, corr in correction_config.items():
-        print('Correcting', corr_name)
-        with xt.tracker._temp_knobs(tracker, corr['ref_with_knobs']):
-            tw_ref = tracker_co_ref.twiss(method='4d', zeta0=0, delta0=0)
+        _print('Correcting', corr_name)
+        with xt.line._temp_knobs(line, corr['ref_with_knobs']):
+            tw_ref = line_co_ref.twiss(method='4d', zeta0=0, delta0=0)
         vary = [xt.Vary(vv, step=1e-9) for vv in corr['vary']]
         targets = []
         for tt in corr['targets']:
             assert isinstance(tt, str), 'For now only strings are supported for targets'
             for kk in ['x', 'px', 'y', 'py']:
-                targets.append(xt.Target(kk, at=tt, value=tw_ref[tt, kk], tol=1e-9))
+                targets.append(xt.Target(kk, at=tt, value=tw_ref[kk, tt], tol=1e-9))
 
-        tracker.match(
+        line.match(
             solver=solver,
             verbose=verbose,
             restore_if_fail=restore_if_fail,
             vary=vary,
             targets=targets,
             twiss_init=xt.OrbitOnly(
-                x=tw_ref[corr['start'], 'x'],
-                px=tw_ref[corr['start'], 'px'],
-                y=tw_ref[corr['start'], 'y'],
-                py=tw_ref[corr['start'], 'py'],
-                zeta=tw_ref[corr['start'], 'zeta'],
-                delta=tw_ref[corr['start'], 'delta'],
+                x=tw_ref['x', corr['start']],
+                px=tw_ref['px', corr['start']],
+                y=tw_ref['y', corr['start']],
+                py=tw_ref['py', corr['start']],
+                zeta=tw_ref['zeta', corr['start']],
+                delta=tw_ref['delta', corr['start']],
             ),
             ele_start=corr['start'], ele_stop=corr['end'])
\ No newline at end of file
